
require 'upload_module'

class InvoicesController < ApplicationController
  include UploadModule
  before_filter :login_required

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update, :print_out ],
         :redirect_to => { :action => :list }
  
  # （自部門の）請求書の一覧
  def list
    div = current_user.division
    raise if !div
    @cdiv = div.control_division
    @invoices = Invoice.find_by_sql [<<EOF, @cdiv.id, @cdiv.id]
(SELECT invoices.* FROM invoices JOIN divisions ON invoices.division_id = divisions.id JOIN control_divisions ON divisions.control_division_id = control_divisions.id
    WHERE control_divisions.id = ?)
UNION
(SELECT invoices.* FROM invoice_details JOIN invoices ON invoice_details.invoice_id = invoices.id JOIN divisions ON invoice_details.division_id = divisions.id JOIN control_divisions ON divisions.control_division_id = control_divisions.id
    WHERE control_divisions.id = ?)
ORDER BY buy_day DESC
OFFSET 0 LIMIT 20
EOF
  end

  # 承認を要するリスト
  def sign_off_list
    @invoice_pages, @invoices = paginate(:invoices,
                            :per_page => 40, :conditions => 'state < 10')
  end
  
  # 承認
  def sign_off
    @invoice = Invoice.find(params[:id])
    Invoice.transaction {
      il = InvoiceLog.new(:invoice => @invoice,
                        :kind => InvoiceLog::SIGNED_OFF,
                        :remarks => "",
                        :done_by => current_user,
                        :done_at => Time.now)
      il.save!
      @invoice.state = 10
      @invoice.save!
    }
    flash[:notice] = '請求書を承認しました。'
    if params[:prev] &&  params[:prev] == 'signoff'
      redirect_to :action => 'sign_off_list'
    else
      redirect_to :action => 'show', :id => @invoice
    end
  end
  
  # 詳細画面
  def show
    @invoice = Invoice.find(params[:id])
    @prev = params[:prev]
  end
  
  # 新しい請求書の画面
  def new
    @invoice = Invoice.new
    @partner = Partner.find(params[:partner])
  end
  
  # 作成
  def create
    @partner = Partner.find(params[:id])
    Invoice.transaction {
      @invoice = Invoice.new(params[:invoice])
      @invoice.attributes = {
        :partner => @partner,
        :division => current_user.division,
        :created_by => current_user.id,
        :state => 0,
        :payment_state => 0
      }
      @invoice.bank_account_id = nil if params[:no_bank]
      
      details = create_details(@invoice.currency.exponent)
      if details.size < 1
        @invoice.errors.add_to_base('エラー：明細が1行もありません。')
        render :action => 'new', :partner => @partner
      else
        details.each {|r| r.save! }
        if @invoice.save
          flash[:notice] = '請求書は正常に登録されました。支払依頼を出力してください。'
          if params[:prev] && params[:prev] == 'list'
            redirect_to :action => 'list'
          else
            redirect_to :action => 'show', :id => @invoice
          end
        else
          render :action => 'new', :partner => @partner
        end
      end
    } # of transaction
  end

  # 編集画面
  def edit
    @invoice = Invoice.find(params[:id])
    @partner = @invoice.partner
  end
  
  # 既存のデータの更新
  def update
    @invoice = Invoice.find(params[:id])
    @partner = @invoice.partner
    Invoice.transaction {
      @invoice.attributes = params[:invoice]
      @invoice.bank_account_id = nil if params[:no_bank]
      
      details = create_details(@invoice.currency.exponent)
      if details.size < 1
        @invoice.errors.add_to_base('エラー：明細が1行もありません。')
        render :action => 'edit'
      else
        InvoiceDetail.delete_all ['invoice_id = ?', @invoice.id]
        details.each {|r| r.save! }
        il = InvoiceLog.new(:invoice => @invoice,
                            :kind => InvoiceLog::MODIFIED,
                            :remarks => params[:message] || "",
                            :done_by => current_user,
                            :done_at => Time.now)
        il.save!

        if @invoice.save
          flash[:notice] = '請求書は正常に更新されました。'
          if params[:prev] &&  params[:prev] == 'signoff'
            redirect_to :action => 'sign_off_list'
          else
            redirect_to :action => 'show', :id => @invoice
          end
        else
          render :action => 'edit'
        end
      end
    } # of transaction
  end
  
  # 明細行のレコードを作る
  def create_details(new_exp)
    result = []
    last = nil
    (0..14).each {|i|      # TODO:決め打ち
      if (pa = params["detail#{i}"])[:amount].to_f != 0.0
        detail = InvoiceDetail.new(
          :invoice => @invoice,
          :account_title => (pa[:account_title_id].to_i != 0 ? AccountTitle.find(pa[:account_title_id]) : last.account_title),
          :division => Division.find(pa[:division_id]),
          :remarks => pa[:remarks],
          :vat_code => pa[:vat_code]
        )
        detail.amount = pa[:amount].to_f * (10 ** new_exp)
        last = detail
        result << detail
      end
    }
    return result
  end
  private :create_details

  # 削除
  def destroy
    inv = Invoice.find(params[:id])
    partner = inv.partner
    Invoice.transaction {
      InvoiceLog.delete_all ['invoice_id = ?', inv.id]
      InvoiceDetail.delete_all ['invoice_id = ?', inv.id]
      inv.destroy
    }
    if params[:prev] && params[:prev] == 'list'
      redirect_to :action => 'list'
    else
      redirect_to :controller => 'partners', :action => 'show', :id => partner
    end
  end
  
  # コピーして新しい請求書を起票
  def copy_new
    @invoice = Invoice.find(params[:id])
    @invoice.buy_day = ((@invoice.buy_day + 1) >> 1) - 1
    @invoice.due_day = ((@invoice.due_day + 1) >> 1) - 1
    @partner = @invoice.partner
    render :action => 'new'
  end

  # 支払依頼を印刷するリスト
  def payment_list
    div = current_user.division.control_division
    @invoices = Invoice.find :all,
       :select => 'invoices.*', 
       :joins => 'JOIN divisions ON invoices.division_id = divisions.id 
          JOIN control_divisions ON divisions.control_division_id = control_divisions.id',
       :conditions => ['control_division_id = ? AND payment_state = ?', div.id, 0]
  end
  
  # プレビュー画面の表示 & 確定
  def print_out
    @invoices = Invoice.find params[:pp], :order => 'partner_id, due_day'
    render :action => 'print_out'
    Invoice.transaction {
      @invoices.each {|inv|
        inv.payment_state = 10  # 印刷済み
        inv.save!
      }
    }
  end
end
